local ezInterrupt = LibStub("AceAddon-3.0"):NewAddon("ezInterrupt", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
local LSM = LibStub("LibSharedMedia-3.0")
local AceGUI = LibStub("AceGUI-3.0")
local init = true
local outputChannel = "NONE"
local msgTags = {
	["[target]"] 	= nil, -- target's name
	["[spell]"] 	= nil, -- spell link of the interrupted ability
	["[interrupt]"] = nil, -- spell link of the interrupt ability
}

LSM:Register("sound", "Shay's Bell", [[Sound\Spells\ShaysBell.ogg]])
LSM:Register("sound", "Simon Chime", [[Sound\Doodad\SimonGame_LargeBlueTree.wav]])
LSM:Register("sound", "Humm", [[Sound\Spells\SimonGame_Visual_GameStart.wav]])
LSM:Register("sound", "Short Circuit", [[Sound\Spells\SimonGame_Visual_BadPress.wav]])
LSM:Register("sound", "ezInterrupt: Beep", [[Interface\Addons\ezInterrupt\media\beep.ogg]])
LSM:Register("sound", "ezInterrupt: Ding", [[Interface\Addons\ezInterrupt\media\ding.ogg]])

local defaults = {
    profile = {
        enableAddon = true,
		enableAnnouncing = true,
		enableWarning = true,
		enableWarningSound = true,
		enableWarningIcon = true,
		enableWhispering = true,
		enableCustomChannel = true,
		enableInterruptCooldownFilter = true,
		tracking = "target",
		whisperTarget = nil,
		customChannel = nil,
		customMessage = "Interrupted [spell]",
		slashSay = true,
		slashParty = true,
		slashRaid = false,
		useInRaid = true,
		useInParty = true,
		useInBattleground = false,
		useInArena = false,
		useOutsideInstance = true,
		soundFile = "ezInterrupt: Ding",
		filterMode = 1, -- 2 = filter by whiteList, 1 = filter by blackList, 0 = don't filter 
		whiteList = {},
		blackList = {},
		warningIconScale = 1,
		warningIconAlpha = 1,
		warningIconOffsetX = 0,
		warningIconOffsetY = 0,
    },
}

local options = {
    name = "ezInterrupt Configuration",
    handler = ezInterrupt,
    type = "group",
    args = {
		enable = {
			type = "toggle",
			name = "Enable Addon",
			width = "full",
			set = function(info, val) ezInterrupt.db.profile.enableAddon = val if ezInterrupt.db.profile.enableAddon then ezInterrupt:OnEnable() else ezInterrupt:OnDisable() end end,
			get = function(info) return ezInterrupt.db.profile.enableAddon end,
			order = 1
		},
		announcing = {
			type = "toggle",
			name = "Announce interrupts",
			width = "full",
			set = function(info, val) ezInterrupt.db.profile.enableAnnouncing = val ezInterrupt:UpdateCombatLogParsing() end,
			get = function(info) return ezInterrupt.db.profile.enableAnnouncing end,
			order = 2
		},
		warning = {
			type = "toggle",
			name = "Spellcast warnings",
			width = "full",
			set = function(info, val) ezInterrupt.db.profile.enableWarning = val ezInterrupt:UpdateCombatLogParsing() end,
			get = function(info) return ezInterrupt.db.profile.enableWarning end,
			order = 3
		},
		output = {
			type = "group",
			name = "Announcement Options",
			order = 5,
			inline = true,
			args = {
				header1 = {
					type = "description",
					name = "Check the output channels that you want to make available to the addon. It will select the appropriate channel automatically and only report once per interrupt. Raid takes priority over Party which takes priority over Say.",
					order = 1
				},
				slashsay = {
					type = "toggle",
					name = "Say",
					width = "half",
					set = function(info, val) ezInterrupt.db.profile.slashSay = val ezInterrupt:UpdateOutputChannel() end,
					get = function(info) return ezInterrupt.db.profile.slashSay end,
					order = 4
				},
				slashparty = {
					type = "toggle",
					name = "Party",
					width = "half",
					set = function(info, val) ezInterrupt.db.profile.slashParty = val ezInterrupt:UpdateOutputChannel() end,
					get = function(info) return ezInterrupt.db.profile.slashParty end,
					order = 3
				},
				slashraid = {
					type = "toggle",
					name = "Raid",
					width = "half",
					set = function(info, val) ezInterrupt.db.profile.slashRaid = val ezInterrupt:UpdateOutputChannel() end,
					get = function(info) return ezInterrupt.db.profile.slashRaid end,
					order = 2
				},
				header2 = {
					type = "description",
					name = "In addition to the above, the addon can also whisper another player when you interrupt or show your interrupts in a custom channel.",
					order = 5
				},
				whispering = {
					type = "toggle",
					name = "Enable Whispering",
					set = function(info, val) ezInterrupt.db.profile.enableWhispering = val end,
					get = function(info) return ezInterrupt.db.profile.enableWhispering end,
					order = 6
				},
				whispertarget = {
					type = "input",
					name = "Which Player?",
					desc = "Input the name of the player you want to whisper to.",
					set = function(info, val) if val == "" then ezInterrupt.db.profile.whisperTarget = nil else ezInterrupt.db.profile.whisperTarget = val end end,
					get = function(info) return ezInterrupt.db.profile.whisperTarget end,
					order = 7
				},
				customchannel = {
					type = "toggle",
					name = "Enable Custom Channel",
					set = function(info, val) ezInterrupt.db.profile.enableCustomChannel = val end,
					get = function(info) return ezInterrupt.db.profile.enableCustomChannel end,
					order = 8
				},
				customchannelfield = {
					type = "input",
					name = "Which Channel?",
					desc = "Insert the number of the custom channel you want to report to. Example: 5",
					set = function(info, val) if val == "" then ezInterrupt.db.profile.customChannel = nil else ezInterrupt.db.profile.customChannel = val end end,
					get = function(info) return ezInterrupt.db.profile.customChannel end,
					order = 9
				},
				header3 = {
					type = "description",
					name = "You can construct a customized interrupt message using the following tags:|n|cFFff9000[target]|r will be replaced by the name of the unit that was interrupted.|n|cFFff9000[spell]|r will be replaced with the interrupted spell.|n|cFFff9000[interrupt]|r will be replaced by the name of your interrupt ability.",
					order = 10
				},
				customMessage = {
					type = "input",
					width = "full",
					name = "Message",
					desc = "Example: Interrupted [target]'s [spell] with [interrupt] >>> Player next! <<<",
					set = function(info, val) if val == "" then ezInterrupt.db.profile.customMessage = "Interrupted [spell]" else ezInterrupt.db.profile.customMessage = val end end,
					get = function(info) return ezInterrupt.db.profile.customMessage end,
					order = 12
				},
			}
		},
		usewhere = {
			type = "group",
			name = "Where should ezInterrupt become active?",
			order = 6,
			inline = true,
			args = {
				useinparty = {
					type = "toggle",
					name = "In party instances",
					width = "full",
					set = function(info, val) ezInterrupt.db.profile.useInParty = val ezInterrupt:UpdateCombatLogParsing() end,
					get = function(info) return ezInterrupt.db.profile.useInParty end,
					order = 3,
				},
				useinraid = {
					type = "toggle",
					name = "In raid instances",
					width = "full",					
					set = function(info, val) ezInterrupt.db.profile.useInRaid = val ezInterrupt:UpdateCombatLogParsing() end,
					get = function(info) return ezInterrupt.db.profile.useInRaid end,
					order = 2
				},
				useOutsideInstance = {
					type = "toggle",
					name = "Outside of instances",
					width = "full",
					set = function(info, val) ezInterrupt.db.profile.useOutsideInstance = val ezInterrupt:UpdateCombatLogParsing() end,
					get = function(info) return ezInterrupt.db.profile.useOutsideInstance end,
					order = 1
				},		
				useinbg = {
					type = "toggle",
					name = "In battlegrounds",
					width = "full",
					set = function(info, val) ezInterrupt.db.profile.useInBattleground = val ezInterrupt:UpdateCombatLogParsing() end,
					get = function(info) return ezInterrupt.db.profile.useInBattleground end,
					order = 4
				},		
				useinarena = {
					type = "toggle",
					name = "In arenas",
					width = "full",
					set = function(info, val) ezInterrupt.db.profile.useInArena = val ezInterrupt:UpdateCombatLogParsing() end,
					get = function(info) return ezInterrupt.db.profile.useInArena end,
					order = 5
				},					
			},
		},
		warningoptions = {
			type = "group",
			name = "Warning Options",
			order = 7,
			inline = true,
			args = {
				enablesound = {
					type = "toggle",
					name = "Play Sound",
					width = "full",
					set = function(info, val) ezInterrupt.db.profile.enableWarningSound = val end,
					get = function(info) return ezInterrupt.db.profile.enableWarningSound end,
					order = 1,
				},
				enablevisual = {
					type = "toggle",
					name = "Show Icon",
					width = "full",					
					set = function(info, val) ezInterrupt.db.profile.enableWarningIcon = val end,
					get = function(info) return ezInterrupt.db.profile.enableWarningIcon end,
					order = 2
				},
				tracking = {
					type = "select",
					name = "Track which unit?",
					desc = "Select the unit that you want see warnings for.",
					style = "dropdown",
					width = "full",			
					values = {["target"]="Your target",["focus"]="Your focus target"},
					set = function(info, val) ezInterrupt.db.profile.tracking = val end,
					get = function(info) return ezInterrupt.db.profile.tracking end,
					order = 3
				},
				header4 = {
					type = "description",
					name = "Tip: warning sounds are played through the master volume channel.",
					order = 5,
				},
				selectsound = {
					type = "select", dialogControl = 'LSM30_Sound',
					name = "Which sound file?",
					style = "dropdown",
					width = "full",
					values = AceGUIWidgetLSMlists.sound,
					set = function(info, val) ezInterrupt.db.profile.soundFile = val end,
					get = function(info) return ezInterrupt.db.profile.soundFile end,
					order = 4
				},	
				filtering = {
					type = "select",
					name = "Spell filtering",
					style = "dropdown",
					width = "full",
					values = {[1]="Disabled",[2]="Blacklist Filtering",[3]="Whitelist Filtering"},
					set = function(info, val) ezInterrupt.db.profile.filterMode = val end,
					get = function(info) return ezInterrupt.db.profile.filterMode end,
					order = 6
				},	
				blacklist = {
					type = "input",
					name = "Blacklist: add/remove",
					desc = "Insert a spell name to add it to the blacklist. Add it again to remove it.",
					get = function(info) return nil end,
					set = function(info, val) ezInterrupt:UpdateFilterList(ezInterrupt.db.profile.blackList, "Blacklist", val) end,
					order = 7
				},	
				whitelist = {
					type = "input",
					name = "Whitelist: add/remove",
					desc = "Insert a spell name to add it to the whitelist. Add it again to remove it.",
					get = function(info) return nil end,
					set = function(info, val) ezInterrupt:UpdateFilterList(ezInterrupt.db.profile.whiteList, "Whitelist", val) end,
					order = 9
				},	
				displayblacklist = {
					type = "execute",
					name = "Display Blacklist",					
					func = function(info) ezInterrupt:showFilterListContents(ezInterrupt.db.profile.blackList, "Blacklist") end,
					order = 8
				},		
					displaywhitelist = {
					type = "execute",
					name = "Display Whitelist",					
					func = function(info) ezInterrupt:showFilterListContents(ezInterrupt.db.profile.whiteList, "Whitelist") end,
					order = 10
				},		
					showtesticon = {
					type = "execute",
					name = "Toggle Example Icon",
					width = "full",
					func = function(info) if WarningIcon:IsVisible() then WarningIcon:Hide() else WarningIcon:Show() end end,
					order = 11
				},		
				iconscale = {
					type = "range",
					name = "Warning Icon Scale",
					min = 0.25,
					max = 3,
					width = "full",
					isPercent = true,
					set = function(info, val) ezInterrupt.db.profile.warningIconScale = val WarningIcon:SetScale(ezInterrupt.db.profile.warningIconScale) end,
					get = function(info) return ezInterrupt.db.profile.warningIconScale end,
					order = 12
				},		
				iconalpha = {
					type = "range",
					name = "Warning Icon Transparency",
					min = 0.1,
					max = 1,
					width = "full",
					isPercent = true,
					set = function(info, val) ezInterrupt.db.profile.warningIconAlpha = val WarningIcon:SetBackdropColor(1, 1, 1 , ezInterrupt.db.profile.warningIconAlpha) end,
					get = function(info) return ezInterrupt.db.profile.warningIconAlpha end,
					order = 13
				},	
				iconX = {
					type = "input",
					name = "Icon X Coordinate",
					desc = "Set the x coordinate of the warning icon on the screen.",
					get = function(info) return ezInterrupt.db.profile.warningIconOffsetX end,
					set = function(info, val) ezInterrupt.db.profile.warningIconOffsetX = val WarningIcon:SetPoint("CENTER", UIParent, "CENTER", ezInterrupt.db.profile.warningIconOffsetX, ezInterrupt.db.profile.warningIconOffsetY) end,
					order = 17
				},
				iconY = {
					type = "input",
					name = "Icon Y Coordinate",
					desc = "Set the y coordinate of the warning icon on the screen.",
					get = function(info) return ezInterrupt.db.profile.warningIconOffsetY end,
					set = function(info, val) ezInterrupt.db.profile.warningIconOffsetY = val WarningIcon:SetPoint("CENTER", UIParent, "CENTER", ezInterrupt.db.profile.warningIconOffsetX, ezInterrupt.db.profile.warningIconOffsetY) end,
					order = 18
				},
			}
		}
	}
}

function ezInterrupt:OnInitialize()
	init = true
	self.db = LibStub("AceDB-3.0"):New("ezInterruptDB", defaults, "Default")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("ezInterrupt", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("ezInterrupt", "ezInterrupt")
	
	WarningIcon = CreateFrame("FRAME", "ezInterruptMainFrame", UIParent)
	WarningIcon:SetFrameStrata("DIALOG")
	WarningIcon:SetPoint("CENTER", UIParent, "CENTER", self.db.profile.warningIconOffsetX, self.db.profile.warningIconOffsetY)
	WarningIcon:SetWidth(64)
	WarningIcon:SetHeight(64)
	WarningIcon:SetClampedToScreen(true)
	WarningIcon:SetScale(self.db.profile.warningIconScale)
	WarningIcon:SetBackdrop({bgFile = "Interface\\Icons\\Spell_Nature_Cyclone"})
	WarningIcon:SetBackdropColor(1, 1, 1 , self.db.profile.warningIconAlpha)
	WarningIcon:Hide()
	
	self.InterruptList = {}
	local class = UnitClass("player")	
		if class == "Warrior" then
		tinsert(self.InterruptList, 6552)	-- Pummel
		tinsert(self.InterruptList, 72) 	-- Shield Bash		
	elseif class == "Druid" then
		tinsert(self.InterruptList, 80964)  -- Skull Bash (Bear)
		tinsert(self.InterruptList, 80965)  -- Skull Bash (Cat)
	elseif class == "Rogue" then
		tinsert(self.InterruptList, 1766)   -- Kick
	elseif class == "Paladin" then
		tinsert(self.InterruptList, 96231)  -- Rebuke
	elseif class == "Deathknight" then
		tinsert(self.InterruptList, 47528)  -- Mind Freeze 
	elseif class == "Shaman" then
		tinsert(self.InterruptList, 57994)  -- Wind Shear
	elseif class == "Mage" then
		tinsert(self.InterruptList, 2139)   -- Counterspell
	elseif class == "Warlock" then
		tinsert(self.InterruptList, 19647)  -- Spell Lock
	elseif class == "Hunter" then
		tinsert(self.InterruptList, 34490)  -- Silencing Shot
		tinsert(self.InterruptList, 50479)  -- Nether Shock (Nether Ray pets)
		tinsert(self.InterruptList, 26090)  -- Pummel (Gorilla pets)
	elseif class == "Priest" then
		tinsert(self.InterruptList, 15487)  -- Silence
	end	
end

-----------------------------
-- experimental code start
-----------------------------
function ezInterrupt:showFilterListContents(list, listname)
	if #list > 0 then
		DEFAULT_CHAT_FRAME:AddMessage("|cFFff9000ez|r|cFFccff1dInterrupt:|r the "..listname.." contains")
		for i,v in ipairs(list) do 
		DEFAULT_CHAT_FRAME:AddMessage(i..". |cFF5dd9f8"..v.."|r")
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("|cFFff9000ez|r|cFFccff1dInterrupt:|r the "..listname.." is empty")
	end
end

-----------------------------
-- experimental code end
-----------------------------

function ezInterrupt:UNIT_SPELLCAST_SUCCEEDED(event, unitID, _, _, _, spellID) 
	if (unitID == "player" or unitID == "pet") and tContains(self.InterruptList, spellID) then
		local timer = ezInterrupt:ScheduleTimer("TimerFinished", 0.5, spellID) -- start preliminary timer
	end
end

function ezInterrupt:TimerFinished(spellID)
	if spellID ~= nil then -- preliminary timer
		local cd = select(2, GetSpellCooldown(spellID))
		cd = cd-0.5 -- subtract the duration of the preliminary timer from the interrupt cooldown
		local cdtimer = ezInterrupt:ScheduleTimer("TimerFinished", cd)
	elseif UnitIsEnemy("player", ezInterrupt.db.profile.tracking) then -- cooldown timer
		local spellName = select(1, UnitCastingInfo(ezInterrupt.db.profile.tracking)) or select(1, UnitChannelInfo(ezInterrupt.db.profile.tracking))
		local immune = select(9, UnitCastingInfo(ezInterrupt.db.profile.tracking)) or select(8, UnitChannelInfo(ezInterrupt.db.profile.tracking))
		if (spellName ~= nil and not immune and self:SpellFilterStatus(spellName, true)) then
			self:StartAlert()
		end
	end
end

function ezInterrupt:IsInterruptReady()
	for i,v in ipairs(self.InterruptList) do
		local ready, noresource = IsUsableSpell(v) -- ready is true if spell is usable; noresource is true if the player lacks energy/rage/etc but could otherwise use it
		if (ready or noresource) and select(2, GetSpellCooldown(v)) == 0 then
			return true
		end
	end
	return false
end

function ezInterrupt:SpellFilterStatus(spellName, ignoreCD)
	if self.db.profile.filterMode == 2 and tContains(self.db.profile.blackList, spellName) then
		return false
	end
	if self.db.profile.filterMode == 3 and not tContains(self.db.profile.whiteList, spellName) then
		return false
	end
	if not ignoreCD and self.db.profile.enableInterruptCooldownFilter and not self:IsInterruptReady() then
		return false
	end
	return true
end

function ezInterrupt:StartAlert(spellName)
	if self.db.profile.enableWarningSound then 
		PlaySoundFile(LSM:Fetch("sound", self.db.profile.soundFile), "MASTER")
	end
	if self.db.profile.enableWarningIcon then
		local icon = select(4, UnitCastingInfo(ezInterrupt.db.profile.tracking)) or select(4, UnitChannelInfo(ezInterrupt.db.profile.tracking))
		WarningIcon:SetBackdrop({bgFile = icon})
		WarningIcon:Show()
	end
end

function ezInterrupt:StopAlert()
	WarningIcon:Hide()
end

-- OnCastEnd handles UNIT_SPELLCAST_STOP and UNIT_SPELLCAST_CHANNEL_STOP
function ezInterrupt:OnCastEnd(event, unitID, spellName)
	if unitID == ezInterrupt.db.profile.tracking and UnitIsEnemy("player", unitID) then
		self:StopAlert()
	end
end

function ezInterrupt:UNIT_SPELLCAST_START(event, unitID, spellName)
	--[[ debug code start --
	if select(9, UnitCastingInfo("target")) == nil then local msg = "error in ezInterrupt:UNIT_SPELLCAST_START. local immune == nil." print(msg) message(msg) end
	-- debug code end ]]--
	if unitID == ezInterrupt.db.profile.tracking and UnitIsEnemy("player", unitID) and not select(9, UnitCastingInfo(ezInterrupt.db.profile.tracking)) and self:SpellFilterStatus(spellName) then 
		self:StartAlert()
	end
end

function ezInterrupt:UNIT_SPELLCAST_CHANNEL_START(event, unitID, spellName)
	--[[ debug code start --
	if select(8, UnitChannelInfo("target")) == nil then local msg = "error in ezInterrupt:UNIT_SPELLCAST_CHANNEL_START. local immune == nil." print(msg) message(msg) end
	-- debug code end ]]--
	if unitID == ezInterrupt.db.profile.tracking and UnitIsEnemy("player", unitID) and not select(8, UnitChannelInfo(ezInterrupt.db.profile.tracking)) and self:SpellFilterStatus(spellName) then 
		self:StartAlert()
	end
end

function ezInterrupt:PLAYER_TARGET_CHANGED()
	if UnitIsEnemy("player", "target") then
		local spellName = select(1, UnitCastingInfo("target")) or select(1, UnitChannelInfo("target"))
		local immune = select(9, UnitCastingInfo("target")) or select(8, UnitChannelInfo("target"))
		if (spellName ~= nil and not immune and self:SpellFilterStatus(spellName)) then
			self:StartAlert()
		else
			self:StopAlert()
		end
	else
		self:StopAlert()
	end
end
-- test this
function ezInterrupt:UNIT_SPELLCAST_INTERRUPTIBLE(event, unitID)
	if unitID == ezInterrupt.db.profile.tracking and UnitIsEnemy("player", unitID) then
		local spellName = select(1, UnitCastingInfo("target")) or select(1, UnitChannelInfo("target"))
		if (spellName ~= nil and self:SpellFilterStatus(spellName)) then
			self:StartAlert()
		end
	end
end
-- test this
function ezInterrupt:UNIT_SPELLCAST_NOT_INTERRUPTIBLE(event, unitID)
	if unitID == ezInterrupt.db.profile.tracking then
		self:StopAlert()
	end
end

function ezInterrupt:UpdateFilterList(table, tableName, spellName) -- add/remove spells from/to blacklist/whitelist
	if spellName ~= nil and spellName ~= "" then
		if tContains(table, spellName) then
			tRemoveByValue(table, spellName)
			DEFAULT_CHAT_FRAME:AddMessage("|cFFff9000ez|r|cFFccff1dInterrupt:|r removed |cFF5dd9f8"..spellName.."|r from "..tableName)
		else 
			tinsert(table, spellName)
			DEFAULT_CHAT_FRAME:AddMessage("|cFFff9000ez|r|cFFccff1dInterrupt:|r added |cFF5dd9f8"..spellName.."|r to "..tableName)
		end	
	else
			DEFAULT_CHAT_FRAME:AddMessage("|cFFff9000ez|r|cFFccff1dInterrupt:|r invalid spell name")
	end
end

function tRemoveByValue(table, value)
	for i,v in ipairs(table) do 
		if v == value then tremove(table, i) end
	end
end

function ReplaceTags(str)
	return msgTags [str]
end

function ezInterrupt:OnEnable()
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED", "UpdateOutputChannel")
	if init then
		init = false
	else
		self:UpdateOutputChannel() 
		self:UpdateCombatLogParsing()
	end
end

function ezInterrupt:OnDisable()
	self:UnregisterAllEvents()		
end

function ezInterrupt:PLAYER_ENTERING_WORLD(event)
	self:UpdateOutputChannel()
	self:UpdateCombatLogParsing()
end

function ezInterrupt:UpdateOutputChannel()
	if self.db.profile.slashRaid and GetNumRaidMembers() > 0 then 
		outputChannel = "RAID"
	elseif self.db.profile.slashParty and ((GetNumPartyMembers() > 0) or (GetNumRaidMembers() > 0)) then
		outputChannel = "PARTY"
	elseif self.db.profile.slashSay then 
		outputChannel = "SAY"
	else
		outputChannel = "NONE"
	end
	--[[ debug code
	print("updated outputChannel to "..outputChannel)
	]]--
end

function ezInterrupt:UpdateCombatLogParsing() -- enables/disables combat log parsing according to instance type and configuration settings
	local instanceType = select(2, IsInInstance())
	if (instanceType == "raid" and self.db.profile.useInRaid) or (instanceType == "party" and self.db.profile.useInParty) or (instanceType == "pvp" and self.db.profile.useInBattleground) or (instanceType == "arena" and self.db.profile.useInArena) or (instanceType == "none" and self.db.profile.useOutsideInstance) then
		if self.db.profile.enableAnnouncing then
			self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		else
			self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		end
		if self.db.profile.enableWarning then
			self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
			self:RegisterEvent("UNIT_SPELLCAST_STOP", "OnCastEnd")
			self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP", "OnCastEnd")
			self:RegisterEvent("UNIT_SPELLCAST_START")
			self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
			self:RegisterEvent("PLAYER_TARGET_CHANGED")
			self:RegisterEvent("UNIT_SPELLCAST_NOT_INTERRUPTIBLE")
			self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTIBLE")
		else
			self:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED")
			self:UnregisterEvent("UNIT_SPELLCAST_START")
			self:UnregisterEvent("UNIT_SPELLCAST_STOP")
			self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START")
			self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
			self:UnregisterEvent("PLAYER_TARGET_CHANGED")
			self:UnregisterEvent("UNIT_SPELLCAST_NOT_INTERRUPTIBLE")
			self:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTIBLE")
		end
	else
		self:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED")
		self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		self:UnregisterEvent("UNIT_SPELLCAST_START")
		self:UnregisterEvent("UNIT_SPELLCAST_STOP")
		self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START")
		self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
		self:UnregisterEvent("PLAYER_TARGET_CHANGED")
		self:UnregisterEvent("UNIT_SPELLCAST_NOT_INTERRUPTIBLE")
		self:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTIBLE")
	end
end

function ezInterrupt:COMBAT_LOG_EVENT_UNFILTERED(_ , _, eventType, sourceGUID, _, _, _, destName, _, spellID, _, _, extraID, ...)
	if eventType == "SPELL_INTERRUPT" and (sourceGUID == UnitGUID("player") or sourceGUID == UnitGUID("pet")) then
		msgTags ["[spell]"] 		= GetSpellLink(extraID)
		msgTags ["[target]"] 		= destName
		msgTags ["[interrupt]"] 	= GetSpellLink(spellID)
		local msg = string.gsub(self.db.profile.customMessage, ".%a+.", ReplaceTags)
		if self.db.profile.enableWhispering and self.db.profile.whisperTarget ~= nil then SendChatMessage(msg, "WHISPER", nil, self.db.profile.whisperTarget ) end
		if self.db.profile.enableCustomChannel and self.db.profile.customChannel ~= nil then SendChatMessage(msg, "CHANNEL", nil, self.db.profile.customChannel ) end	
		if outputChannel ~= "NONE" then SendChatMessage(msg, outputChannel) end
	end
end